/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.perf;

import java.net.URL;
import java.util.Collection;
import java.util.List;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.BusinessArchiveFactory;
import org.ow2.bonita.util.ProcessBuilder;


public abstract class AbstractPerftestCase implements PerfTestCase {

  protected long id;

  public long getId() {
    return this.id;
  }
  public void setId(final long id) {
    this.id = id;
  }

  protected void executeTasks(final RuntimeAPI runtimeAPI, final QueryRuntimeAPI queryRuntimeAPI, 
      final ProcessInstanceUUID instanceUUID,
      final int expected) throws InstanceNotFoundException,
  TaskNotFoundException, IllegalTaskStateException {
    int cpt = expected;
    while (cpt > 0) {
      final Collection<TaskInstance> activities =
        queryRuntimeAPI.getTaskList(instanceUUID, ActivityState.READY);
      if (activities == null || activities.isEmpty()) {
        throw new BonitaRuntimeException("Wrong number of executed tasks. Expected is "
            + expected + " and really executed is: " + (expected - cpt));
      }
      for (final TaskInstance activity : activities) {
        final ActivityInstanceUUID taskUUID = activity.getUUID();
        runtimeAPI.startTask(taskUUID, true);
        runtimeAPI.finishTask(taskUUID, true);
        cpt--;
      }
    }
    if (cpt < 0) {
      throw new BonitaRuntimeException("More tasks than expected(" + expected + ") are available for instance: " + instanceUUID);
    }
    try {
      final Collection<TaskInstance> tasks =
        queryRuntimeAPI.getTaskList(instanceUUID, ActivityState.READY);
      if (tasks != null && tasks.size() > 0) {
        throw new BonitaRuntimeException("More tasks than expected(" + expected
            + ") are available for instance: " + instanceUUID);
      }
    } catch (final InstanceNotFoundException e) {
      // TODO: nothing, all tasks are executed and the instance may be finished
    }
  }

  protected long executeCommonInstance(final int expectedTasks) {
    long endTime = -1;
    ProcessInstanceUUID instanceUUID = null;
    final RuntimeAPI runtimeAPI = AccessorUtil.getAPIAccessor().getRuntimeAPI();
    final QueryRuntimeAPI queryRuntimeAPI = AccessorUtil.getAPIAccessor().getQueryRuntimeAPI();
    try {
      instanceUUID = runtimeAPI.instantiateProcess(getProcessUUID());
      executeTasks(runtimeAPI, queryRuntimeAPI, instanceUUID, expectedTasks);
      endTime = System.currentTimeMillis();
    } catch (final Throwable e) {
      throw new BonitaRuntimeException(e);
    } finally {
      if (instanceUUID != null) {
        try {
          runtimeAPI.deleteProcessInstance(instanceUUID);
        } catch (final InstanceNotFoundException e) {
          throw new BonitaRuntimeException(e);
        } catch (final UndeletableInstanceException e) {
          throw new BonitaRuntimeException(e);
        }
      }
    }
    return endTime;
  }

  public void deploy() {
    final URL xpdlUrl = getXpdlUrl();
    if (xpdlUrl == null) {
      throw new BonitaRuntimeException("Can't find xpdl ressource: xpdlUrl is null");
    }
    final ManagementAPI managementAPI = AccessorUtil.getAPIAccessor().getManagementAPI();
    try {
      ProcessDefinition process = ProcessBuilder.createProcessFromXpdlFile(xpdlUrl);
      managementAPI.deploy(BusinessArchiveFactory.getBusinessArchive(process, null, getClasses().toArray(new Class[]{})));
    } catch (final Exception e) {
      throw new BonitaRuntimeException("Error during deployment", e);
    }
  }

  public void undeploy() {
    try {
      final ManagementAPI managementAPI = AccessorUtil.getAPIAccessor().getManagementAPI();
      managementAPI.disable(getProcessUUID());
      managementAPI.deleteProcess(getProcessUUID());
    } catch (final Exception e) {
      throw new BonitaRuntimeException("Error during undeployment", e);
    }
  }

  private ProcessDefinitionUUID getProcessUUID() throws ProcessNotFoundException {
    final String processId = getProcessId();
    final ProcessDefinition processDefinition =
      AccessorUtil.getAPIAccessor().getQueryDefinitionAPI().getLastProcess(processId);
    return processDefinition.getUUID();
  }

  protected abstract URL getXpdlUrl();
  protected abstract List<Class< ? >> getClasses();
}


